%COMPUTEMAPS Compute weights (cosine map) and mask map
%
%  [W, MASKMAP, MASKMAPLIMITED, HDANGLES] = COMPUTEMAPS(A)
%  computes weights W (cosine map) favoring lower elevations,
%  mask map MASKMAP masking unmeasurable directions,
%  limited mask map MASKMAPLIMITED masking elevations over
%  75 degrees in addition and list of all angles HDANGLES
%  in matrix A.
%
%  [W, MASKMAP, MASKMAPLIMITED, HDANGLES] = COMPUTEMAPS(A, E, HDS)
%  enables choosing value of parameter epsilon E (0.001)
%  and HDS (5) specifying step in half-diff parameterization.
%  Default values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [w, MaskMap, MaskMapLimited, hdAngles] = computeMaps(A, e, hds)

if (~exist('e','var') || isempty(e))
    e = 0.001;
end
if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

fprintf('Computing maps... ');

d2r = pi/180;
r2d = 180/pi;

ntd = floor(89/hds)+1;
nth = floor(89/hds)+1;
npd = floor(179/hds)+1;
thi = (0:nth-1)*hds;

if (exist('A','var') && ~isempty(A))
    MaskMap = all(A > 0, 2);
else
    MaskMap = true(npd*ntd*nth,1);
end

hdAngles = zeros(npd*ntd*nth,4);
hdAngles(:,1) = reshape(repmat(thi,npd*ntd,1),npd*ntd*nth,1);
hdAngles(:,2) = zeros(npd*ntd*nth,1);
hdAngles(:,3) = repmat(reshape(repmat(0:hds:89,npd,1),npd*ntd,1),nth,1);
hdAngles(:,4) = repmat((0:hds:hds*(npd-1))',ntd*nth,1);

hd2 = hdAngles;

id = hd2(:,1) == 0;
hd2(id,2) = 0;
id = hd2(:,3) == 0;
hd2(id,4) = 0;

hd2 = hd2*d2r;

[tl, ~, tv, ~] = half2std(hd2(:,1), hd2(:,2), hd2(:,3), hd2(:,4));

w = max(cos(tl).*cos(tv), e);

below_horizon = tl>=pi/2 | tv>=pi/2;

MaskMap = MaskMap & ~below_horizon;

MaskMapLimited = MaskMap & (tl*r2d <= 75 & tv*r2d <= 75);

w = w(MaskMap);

fprintf('done.\n');
