%COMPUTEOPTIMALDIRECTIONS Compute N optimal directions to measure new BRDF
%
%  DIRECTIONS = COMPUTEOPTIMALDIRECTIONS(Q, N, MASKMAP) computes N optimal
%  DIRECTIONS to measure new BRDF, based on weighted principal components Q
%  allowing to select any direction not masked by MASKMAP.
%
%  DIRECTIONS = COMPUTEOPTIMALDIRECTIONS(Q, N, MASKMAP, MASKMAPLIMITED)
%  computes N optimal DIRECTIONS to measure new BRDF, based on weighted
%  principal components Q allowing to select any direction not masked by
%  MASKMAPLIMITED.
%
%  COMPUTEOPTIMALDIRECTIONS(Q, N, MASKMAP, [], DIRECTIONSDIR) computes N
%  optimal directions to measure new BRDF, based on weighted principal
%  components Q allowing to select any direction not masked by MASKMAP and
%  saving the directions to file in directory specified by DIRECTIONSDIR.
%
%  COMPUTEOPTIMALDIRECTIONS(Q, N, MASKMAP, MASKMAPLIMITED, DIRECTIONSDIR)
%  computes N optimal directions to measure new BRDF, based on weighted
%  principal components Q allowing to select any direction not masked by
%  MASKMAPLIMITED and saving the directions to file in directory specified
%  by DIRECTIONSDIR.
%
%  DIRECTIONS = COMPUTEOPTIMALDIRECTIONS(..., RANDOMSTARTS, TRIES, MAXITER,
%                                        RSIZE, FASTUPDATE, HDS, VERBOSE)
%               COMPUTEOPTIMALDIRECTIONS(..., RANDOMSTARTS, TRIES, MAXITER,
%                                        RSIZE, FASTUPDATE, HDS, VERBOSE)
%  enables to specify number of random starts RANDOMSTARTS (10) of the
%  algorithm, number of tries TRIES (10) of each run of the algorithm,
%  maximum number of iterations MAXITER (1000) of the optimization
%  procedure, radius RSIZE (3) of volume inspected by each iteration
%  of the optimization procedure, whether use fast computation of condition
%  numbers FASTUPDATE (0), step in half-diff parameterization HDS (5)
%  and whether and how much information should be printed
%  VERBOSE (0, 1(default), 2).
%  Default and possible values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function directions = computeOptimalDirections(Q, n, MaskMap, MaskMapLimited, directionsDir, randomStarts, tries, maxIter, rSize, fastUpdate, hds, verbose)

k = min(n,size(Q,2));
if (exist('MaskMapLimited','var') && ~isempty(MaskMapLimited))
    Q0 = zeros(length(MaskMap),k);
    Q0(MaskMap,:) = Q(:,1:k);
    Q = Q0(MaskMapLimited,:);
    MaskMap = MaskMapLimited;
end
if (exist('directionsDir','var') && ~isempty(directionsDir))
    if (~exist(directionsDir,'dir'))
        mkdir(directionsDir);
    end
end
if (~exist('randomStarts','var') || isempty(randomStarts))
    randomStarts = 10;
end
if (~exist('tries','var') || isempty(tries))
    tries = 10;
end
if (~exist('maxIter','var') || isempty(maxIter))
    maxIter = 1000;
end
if (~exist('rSize','var') || isempty(rSize))
    rSize = 3;
end
if (~exist('fastUpdate','var') || isempty(fastUpdate))
    fastUpdate = 0;
end
if (~exist('hds','var') || isempty(hds))
    hds = 5;
end
if (~exist('verbose','var') || isempty(verbose))
    verbose = 1;
end


points = [];

if (n == 1)
    [~,points] = max(sum(Q(:,1:k).^2,2));
    if (exist('directionsDir','var') && ~isempty(directionsDir))
        directions = mapIdxToAngle(points, MaskMap, hds); %#ok<NASGU>
        save(fullfile(directionsDir,sprintf('directions%.4d.mat',n)),'directions');
    end
else
    for start = 1:randomStarts
        
        points2 = randperm(size(Q,1))';
        points2 = points2(1:n);
        
        for t = 1:tries
            
            t1 = tic;

            [points3, c] = optimizeLocations(Q, MaskMap, points2, maxIter, 0.001, rSize, fastUpdate, hds, verbose>=2);
            
            if ((start == 1 && t == 1) || c < bestC)
                bestC = c;
                points = points3;
                
                if (exist('directionsDir','var') && ~isempty(directionsDir))
                    directions = mapIdxToAngle(points, MaskMap, hds); %#ok<NASGU>
                    save(fullfile(directionsDir,sprintf('directions%.4d.mat',n)),'directions');
                end
            end
            
            if (verbose)
                fprintf('start %.3d, try %.3d, best c = %.5f, current c = %.5f, time %.1f s\n', start, t, bestC, c, toc(t1));
            end
        end
        
    end
end

directions = mapIdxToAngle(points, MaskMap, hds);
