%HALF2STD Convert direction in half-diff parameterization
%  to standard parameterization
%
%  [TL,PL,TV,PV] = HALF2STD(TH,PH,TD,PD) converts direction
%  in half-diff parameterization specified by TH, PH, TD, PD
%  angles into standard parameterization using TL, PL, TV, PV
%  angles.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [tl,pl,tv,pv] = half2std(th,ph,td,pd)

n = size(th,1);

half = spherical2xyz(th, ph);
diff = spherical2xyz(td, pd);

bi_normal = repmat([0.0, 1.0, 0.0], n, 1);
normal = repmat([0.0, 0.0, 1.0], n, 1);
temp = rotate_vector(diff, bi_normal, th);
in = rotate_vector(temp, normal, ph);
in = normalize_vector(in);

out = rotate_vector(in, half, repmat(pi,n,1));
out = normalize_vector(out);

tl = acos(in(:,3));
pl = atan2(in(:,2), in(:,1));
tv = acos(out(:,3));
pv = atan2(out(:,2), out(:,1));

pv(pv < 0) = pv(pv < 0) + 2*pi;
pv(pv >= 2*pi) = pv(pv >= 2*pi) - 2*pi;
pl(pl < 0) = pl(pl < 0) + 2*pi;
pl(pl >= 2*pi) = pl(pl >= 2*pi) - 2*pi;

end

% rotate vector along one axis
function out = rotate_vector(vector, axis, angle)

cos_ang = cos(angle);
sin_ang = sin(angle);

out = vector .* repmat(cos_ang, 1, 3);

temp = dot(axis, vector, 2);
temp = temp.*(1.0-cos_ang);

out = out + axis.*repmat(temp,1,3);

c = cross(axis, vector, 2);

out = out + c.*repmat(sin_ang,1,3);

end

% convert spherical coordinates to Euclidean
function out = spherical2xyz(theta, phi)

out(:,3) = cos(theta);
xy = sin(theta);
out(:,1) = xy.*cos(phi);
out(:,2) = xy.*sin(phi);
out = normalize_vector(out);

end

% normalize vector
function out = normalize_vector(in)

out = in./repmat(sqrt(sum(in.^2,2)),1,3);

end
