%OPTIMIZELOCATIONS Optimize given directions to minimize condition number
%
%  [ROWS, BESTC] = OPTIMIZELOCATIONS(Q, MASKMAP, ROWS, MAXITER, CONVPERC,
%  RSIZE, FASTUPDATE) optimizes given ROWS to minimize condition number
%  returned in BESTC. Q are weighted principal components, MASKMAP
%  specifies masked directions, MAXITER specifies maximum number of
%  iterations, CONVPERC specifies what is considered as convergence, RSIZE
%  is radius of volume inspected by each iteration and FASTUPDATE specifies
%  whether use fast computation of condition numbers.
%
%  [ROWS, BESTC] = OPTIMIZELOCATIONS(..., HDS) enables to specify
%  step in half-diff parameterization HDS (5).
%  
%  [ROWS, BESTC] = OPTIMIZELOCATIONS(..., VERBOSE) enables to specify
%  whether to print optimization information.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [rows, bestC] = optimizeLocations(Q, MaskMap, rows, maxIter, convPerc, rSize, fastUpdate, hds, verbose)

if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

n = length(rows);
k = min(n,size(Q,2));

dirs = -rSize:rSize;
l = 2*rSize+1;
l3 = l*l*l;
gradDirs(:,1) = repmat(dirs', l*l, 1);
gradDirs(:,2) = repmat(reshape(repmat(dirs, l, 1), l*l, 1), l, 1);
gradDirs(:,3) = reshape(repmat(dirs, l*l, 1), l*l*l, 1);

ntd = floor(89/hds)+1;
nth = floor(89/hds)+1;
npd = floor(179/hds)+1;

bounds = [npd,ntd,nth];
b = repmat(bounds,[l3,1]);

bestC = cond(Q(rows,1:k));

for iter = 1:maxIter
    
    iterC = bestC;
    selOrder = randperm(n);    
    
    for i = selOrder
        p = rowTo3D(rows(i), MaskMap, hds);
        r = rows;
        r(i) = [];
        
        p1 = repmat(p,l3,1) + gradDirs;
        p1(p1(:,1)<0,1) = b(p1(:,1)<0,1)+p1(p1(:,1)<0,1);
        p1(p1(:,2)<0,2) = 0;
        p1(p1(:,3)<0,3) = 0;
        p1(p1(:,1)>=b(:,1),1) = p1(p1(:,1)>=b(:,1),1)-b(p1(:,1)>=b(:,1),1);
        p1(p1(:,2)>=b(:,2),2) = b(p1(:,2)>=b(:,2),2)-1;
        p1(p1(:,3)>=b(:,3),3) = b(p1(:,3)>=b(:,3),3)-1;
        
        rn = idx3DtoRow(p1, MaskMap, hds);
        rn(any(repmat(rn,1,n-1) == repmat(r',l3,1),2)) = 0;
        rn((l3-1)/2+1) = 0;
        rn(rn==0) = [];
        rn = unique(rn);
        
        if (fastUpdate)
            A = Q(r,1:k);
            c = conditionNumber(A'*A, Q(rn,1:k)');            
        else
            c = zeros(1,size(rn,1));
            for j = 1:size(rn,1);
                c(j) = cond(Q([r; rn(j)],1:k));
            end
        end        
               
        [c,j] = min(c);
        if (c < bestC)
            bestC = c;
            rows(i) = rn(j);
        end
    end
    
    if (exist('verbose','var') && ~isempty(verbose) && verbose > 0)
        fprintf('iter = %.3d, c = %.6f, reduction = %.4f%%\n', iter, bestC, (iterC-bestC)/iterC*100);
    end
    
    if ((iterC-bestC)/iterC*100 <= convPerc)
        break;
    end
    
end

bestC = cond(Q(rows,1:k));
