%READBRDFDATA Read BRDF data from mat files into matrix A
%
%  A = READBRDFDATA() reads data from './data' with default
%  parameters.
%
%  A = READBRDFDATA(DBPATH, STEPS, BOUNDS, HDS) reads data
%  from directory specified by DBPATH ('./data').
%  Data should be in standard parameterization
%  with steps between consecutive elevations and
%  azimuths specified by STEPS ([15 7.5 15 7.5])
%  in order theta_i, phi_i, theta_v, phi_v
%  and their maximal values specified by BOUNDS
%  ([75 360 75 360]). Data are converted into half-diff
%  parameterization with step specified by HDS (5).
%  Default values in brackets.
%  
%  BRDF data can be downloaded in Matlab format from
%  UTIA BRDF Database at http://btf.utia.cas.cz/.
%  Please extract data into directory specified by DBpath
%  (default value './data').
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function A = readBRDFdata(DBpath, steps, bounds, hds)

A = [];

if (~exist('DBpath','var') || isempty(DBpath))
    DBpath = './data';
end
if (~exist(DBpath,'dir'))
    fprintf('Data not found! Please download UTIA BRDF Database from http://btf.utia.cas.cz/ in Matlab format and extract it into %s.\n', DBpath);
    return;
end

if (~exist('steps','var') || isempty(steps))
    steps = [15 7.5 15 7.5];
end
if (~exist('bounds','var') || isempty(bounds))
    bounds = [75 360 75 360];
end
if (~exist('hds','var') || isempty(hds))
    hds = 5;
end


% make material list
d = dir(DBpath);
count = 0;
L = cell(150,1);
for i=1:numel(d)
    if (~d(i).isdir && numel(d(i).name) >= 8 && strcmp(d(i).name(end-3:end),'.mat'))
        count = count + 1;
        L{count,1} = d(i).name;
    end
end;
L(count+1:end) = [];
if (numel(L) == 0)
    fprintf('Data not found! Please download UTIA BRDF Database from http://btf.utia.cas.cz/ in Matlab format and extract it into %s.\n', DBpath);
    return;
elseif (numel(L) < 150)
    fprintf('Not all data found! You can download UTIA BRDF Database from http://btf.utia.cas.cz/ in Matlab format and extract it into %s.\n', DBpath);
end

% constants related to parameterization
d2r = pi/180;
r2d = 180/pi;

th = 0:hds:90-hds;
ph = 0:hds:360-hds;
td = 0:hds:90-hds;
pd = 0:hds:180-hds;

nth = numel(th);
nph = numel(ph);
ntd = numel(td);
npd = numel(pd);

hd = zeros(nth*nph*ntd*npd, 4);

hd(:,1) = repmat(reshape(repmat(th, ntd*npd, 1), nth*ntd*npd, 1), nph, 1);
hd(:,2) = reshape(repmat(ph, nth*ntd*npd, 1), nth*nph*ntd*npd, 1);
hd(:,3) = repmat(reshape(repmat(td, npd, 1), ntd*npd, 1), nph*nth, 1);
hd(:,4) = repmat(pd', nph*nth*ntd, 1);

hd(hd(:,1)==0,2) = 0;

% convert from half-diff parameterization to standard spherical
% parameterization
[tl, pl, tv, pv] = half2std(hd(:,1)*d2r, hd(:,2)*d2r, hd(:,3)*d2r, hd(:,4)*d2r);

% constants related to interpolation
std0 = [tl, pl, tv, pv]*r2d;

n = size(std0,1);
std0 = std0./repmat(steps,n,1);
bs = bounds./steps;
bs1 = bs+[1 0 1 0];

std{1} = floor(std0);
stdw{2} = std0-std{1};
stdw{1} = 1-stdw{2};
std{2} = std{1}+1;
std{1}(std{1}(:,1)>bs(1),1) = bs(1);
std{1}(std{1}(:,3)>bs(3),3) = bs(3);
std{2}(std{2}(:,1)>bs(1),1) = bs(1);
std{2}(std{2}(:,2)>=bs(2),2) = std{2}(std{2}(:,2)>=bs(2),2) - bs(2);
std{2}(std{2}(:,3)>bs(3),3) = bs(3);
std{2}(std{2}(:,4)>=bs(4),4) = std{2}(std{2}(:,4)>=bs(4),4) - bs(4);

% interpolation indices and weights
idx = zeros(n,16);
w = zeros(n,16);
for a1 = 1:2
    for a2 = 1:2
        for a3 = 1:2
            for a4 = 1:2
                i = (((a1-1)*2 + a2-1)*2 + a3-1)*2 + a4;
                idx(:,i) = ((std{a1}(:,3)*bs1(4) + std{a2}(:,4))*bs1(1) + std{a3}(:,1))*bs1(2) + std{a4}(:,2) + 1;
                w(:,i) = stdw{a1}(:,3) .* stdw{a2}(:,4) .* stdw{a3}(:,1) .* stdw{a4}(:,2);                
            end
        end
    end
end

A = zeros(npd*ntd*nth, 3*nph*numel(L));

fprintf('Loading data, ');
text = '';
for matNum = 1:numel(L)
    back = repmat('\b',1,numel(text)-1);
    text = sprintf('%d%%%% done.', round(matNum/numel(L)*100));
    fprintf(sprintf('%s%s', back, text));
    
    load(fullfile(DBpath, L{matNum}), 'BRDF');
    BRDF = reshape(BRDF, bs1(1)*bs1(2)*bs1(3)*bs1(4), 3);
    % interpolation
    Y = zeros(n,3);
    for c = 1:3
        for i = 1:16
            Y(:,c) = Y(:,c) + BRDF(idx(:,i),c) .* w(:,i);
        end
    end
    A(:,(matNum-1)*3*nph+(1:3*nph)) = reshape(permute(reshape(Y, [npd*ntd*nth, nph, 3]), [1 3 2]), npd*ntd*nth, 3*nph);    
end
fprintf('\n%d materials loaded.\n', numel(L));

A(A <= eps) = eps;
