%RECONSTRUCT Reconstruct BRDF data based on sparse measurement
%
%  [A, C] = RECONSTRUCT(W, MASKMAP, F_MED_W, MI, Q, DIRECTIONS, VALUES)
%  reconstructs measured BRDF based on sparse samples. Weights W (cosine
%  map) favors lower elevations. Mask map MASKMAP masks unmeasurable
%  directions. F_MED_W is median isotropic BRDF weighted by weights W.
%  MI is mean mapped isotropic BRDF. Q are weighted principal components.
%  DIRECTIONS are measured directions and VALUES are measured values.
%
%  [A, C] = RECONSTRUCT(..., E, ETA, LAMBDA, HDS) enables to specify
%  parameter epsilon E (0.001), parameter ETA (40), parameter LAMBDA (0.1)
%  and half-diff parameterization step HDS (5).
%  Default values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [A, C] = reconstruct(w, MaskMap, f_med_w, mi, Q, directions, values, e, eta, lambda, hds)

if (~exist('e','var') || isempty(e))
    e = 0.001;
end
if (~exist('eta','var') || isempty(eta))
    eta = 40;
end
if (~exist('lambda','var') || isempty(lambda))
    lambda = 0.1;
end
if (~exist('hds','var') || isempty(hds))
    hds = [];
end

[~, idxValid] = mapAngleToIdx(directions, MaskMap, hds);

% get only valid indices
id = idxValid >= 1 & idxValid <= size(w,1);
idxValid = idxValid(id);
values = values(id,:);

% size of data
[n,m] = size(values);
k = min(n,size(Q,2));

% map data, eq 1
X1 = log((values.*repmat(w(idxValid),1,m) + e)./repmat(f_med_w(idxValid)+e,1,m));

% eq 4
Q1 = Q(idxValid,1:k);
mi1 = mi(idxValid);
C = (Q1'*Q1+eta*eye(k))\Q1' * (X1-repmat(mi1,1,m));

% eq 5
if (lambda > 0)
    angles_all = mapIdxToAngle((1:size(Q,1))', MaskMap, hds);
    id_th0 = angles_all(:,3) == 0;
    P = Q(id_th0,1:k);
    M = eye(m/3)-ones(m/3)/(m/3);
    opts = optimoptions(@fminunc, 'Algorithm', 'quasi-newton', 'Display', 'notify', 'MaxFunEvals', 1e6, 'GradObj', 'on', 'MaxIter', 1000);
    for i = 1:3
        C0 = C(:,i:3:end);
        fprintf(' initial error %f\n', objFun(C(:,i:3:end), Q1, X1(:,i:3:end)-repmat(mi1,1,m/3), P, M, eta, lambda));
        f = @(C1)objFun(C1, Q1, X1(:,i:3:end)-repmat(mi1,1,m/3), P, M, eta, lambda);
        C(:,i:3:end) = fminunc(f, C0, opts);
        fprintf('   final error %f\n', objFun(C(:,i:3:end), Q1, X1(:,i:3:end)-repmat(mi1,1,m/3), P, M, eta, lambda));
    end
end

% eq 3
X = Q(:,1:k)*C + repmat(mi,1,m);

% unmap data
A = -ones(length(MaskMap),m);
A(MaskMap,:) = (exp(X).*repmat(f_med_w+e,1,m)-e)./repmat(w,1,m);
