%SAVEBRDFDATA Return or save BRDF data from matrix into mat file
%
%  BRDF = SAVEBRDFDATA(A) returns BRDF data.
%
%  SAVEBRDFDATA(A, FILENAME) saves BRDF data into mat file.
%
%  BRDF = SAVEBRDFDATA(A, FILENAME, STEPS, BOUNDS, HDS) returns and saves
%  BRDF data. Data will be in standard parameterization
%  with steps between consecutive elevations and
%  azimuths specified by STEPS ([15 7.5 15 7.5])
%  in order theta_i, phi_i, theta_v, phi_v
%  and their maximal values specified by BOUNDS
%  ([75 360 75 360]). Input data A are in half-diff
%  parameterization with step specified by HDS (5).
%  Default values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function BRDF = saveBRDFdata(A, filename, steps, bounds, hds)

if (~exist('filename','var'))
    filename = [];
end
if (~exist('steps','var') || isempty(steps))
    steps = [15 7.5 15 7.5];
end
if (~exist('bounds','var') || isempty(bounds))
    bounds = [75 360 75 360];
end
if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

d2r = pi/180;
r2d = 180/pi;

boundsHD = [90-hds 360 90-hds 180];
bs = boundsHD/hds;
bs1 = bs + [1 0 1 0];

t = 0:steps(1):bounds(1);
ntl = bounds(1)/steps(1) + 1;
npl = bounds(2)/steps(2);
ntv = bounds(3)/steps(3) + 1;
npv = bounds(4)/steps(4);
sp = 360/npl;

std = zeros(ntl*npl*ntv*npv, 4);
std(:,1) = repmat(reshape(repmat(t,npl,1),npl*ntl,1),npv*ntv,1);
std(:,2) = repmat((0:sp:360-sp)',ntl*npv*ntv,1);
std(:,3) = reshape(repmat(t,npl*ntl*npv,1),ntl*npl*ntv*npv,1);
std(:,4) = repmat(reshape(repmat(0:sp:360-sp,npl*ntl,1),npl*ntl*npv,1),ntv,1);
std(std(:,1)==0,2) = 0;
std(std(:,3)==0,4) = 0;

[th, ph, td, pd] = std2half(std(:,1)*d2r, std(:,2)*d2r, std(:,3)*d2r, std(:,4)*d2r);

hd0 = [th, ph, td, pd]*r2d;
hd0 = hd0/hds;

hd{1} = floor(hd0);
hdw{2} = hd0-hd{1};
hdw{1} = 1-hdw{2};
hd{2} = hd{1}+1;
hd{1}(hd{1}(:,1)>bs(1),1) = bs(1);
hd{1}(hd{1}(:,3)>bs(3),3) = bs(3);
hd{1}(hd{1}(:,4)>=bs(4),4) = hd{1}(hd{1}(:,4)>=bs(4),4) - bs(4);

hd{2}(hd{2}(:,1) >bs(1),1) = bs(1);
hd{2}(hd{2}(:,2)>=bs(2),2) = hd{2}(hd{2}(:,2)>=bs(2),2) - bs(2);
hd{2}(hd{2}(:,3) >bs(3),3) = bs(3);
hd{2}(hd{2}(:,4)>=bs(4),4) = hd{2}(hd{2}(:,4)>=bs(4),4) - bs(4);
hd{2}(hd{2}(:,4)>=bs(4),4) = hd{2}(hd{2}(:,4)>=bs(4),4) - bs(4);

n = size(hd0,1);

idx = zeros(n,16);
w = zeros(n,16);

for a1 = 1:2
    for a2 = 1:2
        for a3 = 1:2
            for a4 = 1:2
                i = (((a1-1)*2 + a2-1)*2 + a3-1)*2 + a4;
                
                hd1 = [hd{a1}(:,1) hd{a2}(:,2) hd{a3}(:,3) hd{a4}(:,4)];

                id = hd1(:,1) == 0;
                hd1(id,4) = hd1(id,4) + hd1(id,2);
                hd1(id,2) = 0;
                hd1(hd1(:,4)>=bs(4),4) = hd1(hd1(:,4)>=bs(4),4) - bs(4);
                hd1(hd1(:,4)>=bs(4),4) = hd1(hd1(:,4)>=bs(4),4) - bs(4);               
                
                idx(:,i) = ((hd1(:,2)*bs1(1) + hd1(:,1))*bs1(3) + hd1(:,3))*bs1(4) + hd1(:,4) + 1;
                w(:,i) = hdw{a1}(:,1) .* hdw{a2}(:,2) .* hdw{a3}(:,3) .* hdw{a4}(:,4);                
            end
        end
    end
end

th = 0:hds:90-hds;
ph = 0:hds:360-hds;
td = 0:hds:90-hds;
pd = 0:hds:180-hds;

nth = numel(th);
nph = numel(ph);
ntd = numel(td);
npd = numel(pd);

A = reshape(permute(reshape(A, [npd*ntd*nth, 3, nph]),[1 3 2]), npd*ntd*nth*nph, 3);

BRDF = zeros(npl*ntl*npv*ntv, 3);
for c = 1:3
    for i = 1:16
        BRDF(:,c) = BRDF(:,c) + A(idx(:,i),c) .* w(:,i);
    end
end

BRDF = reshape(BRDF, [npl*ntl, npv*ntv, 3]);

if (~isempty(filename))
    save(filename, 'BRDF');
end
