%STD2HALF Convert direction in standard parameterization
%  to half-diff parameterization
%
%  [TH,PH,TD,PD] = STD2HALF(TL,PL,TV,PV) converts direction
%  in standard parameterization specified by TL, PL, TV, PV
%  angles into half-diff parameterization using TH, PH, TD, PD
%  angles.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [th,ph,td,pd] = std2half(tl,pd,tv,ph)

n = size(tl,1);

% compute in vector
in = spherical2xyz(tl,pd);

% compute out vector
out = spherical2xyz(tv,ph);

% compute halfway vector
half = normalize_vector(in+out);
th = acos(half(:,3));
ph = atan2(half(:,2), half(:,1));
ph(th < 0.001) = 0;

% compute diff vector
bi_normal = repmat([0.0, 1.0, 0.0], n, 1);
normal = repmat([0.0, 0.0, 1.0], n, 1);
temp = rotate_vector(in, normal, -ph);
diff = rotate_vector(temp, bi_normal, -th);
diff = normalize_vector(diff);
td = acos(diff(:,3));
pd = atan2(diff(:,2), diff(:,1));

ph(ph < 0) = ph(ph < 0) + 2*pi;
ph(ph >= 2*pi) = ph(ph >= 2*pi) - 2*pi;
pd(pd < 0) = pd(pd < 0) + 2*pi;
pd(pd >= 2*pi) = pd(pd >= 2*pi) - 2*pi;

end


% rotate vector along one axis
function out = rotate_vector(vector, axis, angle)

cos_ang = cos(angle);
sin_ang = sin(angle);

out = vector .* repmat(cos_ang, 1, 3);

temp = dot(axis, vector, 2);
temp = temp.*(1.0-cos_ang);

out = out + axis.*repmat(temp,1,3);

c = cross(axis, vector, 2);

out = out + c.*repmat(sin_ang,1,3);

end

% convert spherical coordinates to Euclidean
function out = spherical2xyz(theta, phi)

out(:,3) = cos(theta);
xy = sin(theta);
out(:,1) = xy.*cos(phi);
out(:,2) = xy.*sin(phi);
out = normalize_vector(out);

end

% normalize vector
function out = normalize_vector(in)

out = in./repmat(sqrt(sum(in.^2,2)),1,3);

end
